# Generated by Django 4.0.7 on 2022-08-18 12:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("structures", "0042_migrate_sender_category_other"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="structureservice",
            name="name_de",
        ),
        migrations.RemoveField(
            model_name="structureservice",
            name="name_ko",
        ),
        migrations.RemoveField(
            model_name="structureservice",
            name="name_ru",
        ),
        migrations.RemoveField(
            model_name="structureservice",
            name="name_zh",
        ),
    ]
