# Generated by Django 4.0.3 on 2022-03-02 18:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("structures", "0035_add_more_notifications_types"),
    ]

    operations = [
        migrations.AlterField(
            model_name="owner",
            name="ping_groups",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Groups to be pinged for each notification. ",
                related_name="+",
                to="auth.group",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="ping_groups",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Groups to be pinged for each notification - ",
                related_name="+",
                to="auth.group",
            ),
        ),
    ]
